@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013, 2015, 2016 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,12 +67,15 @@ void FavoritesPage::add(Launcher* launcher)
 		return;
 	}
 
+	launcher->set_flag(Launcher::FavoriteFlag, true);
+
 	// Append to list of items
 	GtkListStore* store = GTK_LIST_STORE(get_view()->get_model());
 	gtk_list_store_insert_with_values(
 			store, NULL, G_MAXINT,
 			LauncherView::COLUMN_ICON, launcher->get_icon(),
 			LauncherView::COLUMN_TEXT, launcher->get_text(),
+			LauncherView::COLUMN_TOOLTIP, launcher->get_tooltip(),
 			LauncherView::COLUMN_LAUNCHER, launcher,
 			-1);
 }
@@ -81,6 +84,11 @@ void FavoritesPage::add(Launcher* launcher)
 
 void FavoritesPage::remove(Launcher* launcher)
 {
+	if (launcher)
+	{
+		launcher->set_flag(Launcher::FavoriteFlag, false);
+	}
+
 	GtkTreeModel* model = GTK_TREE_MODEL(get_view()->get_model());
 	GtkListStore* store = GTK_LIST_STORE(model);
 	GtkTreeIter iter;
@@ -109,6 +117,15 @@ void FavoritesPage::set_menu_items()
 	g_signal_connect_slot(model, "row-inserted", &FavoritesPage::on_row_inserted, this);
 	g_signal_connect_slot(model, "row-deleted", &FavoritesPage::on_row_deleted, this);
 	g_object_unref(model);
+
+	for (std::vector<std::string>::size_type i = 0, end = wm_settings->favorites.size(); i < end; ++i)
+	{
+		Launcher* launcher = get_window()->get_applications()->get_application(wm_settings->favorites[i]);
+		if (launcher)
+		{
+			launcher->set_flag(Launcher::FavoriteFlag, true);
+		}
+	}
 }
 
 //-----------------------------------------------------------------------------
@@ -126,17 +143,19 @@ void FavoritesPage::extend_context_menu(GtkWidget* menu)
 	GtkWidget* menuitem = gtk_separator_menu_item_new();
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 	menuitem = gtk_image_menu_item_new_with_label(_("Sort Alphabetically A-Z"));
-	GtkWidget* image = gtk_image_new_from_stock(GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
+	GtkWidget* image = gtk_image_new_from_icon_name("view-sort-ascending", GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
 	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &FavoritesPage::sort_ascending, this);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
 	menuitem = gtk_image_menu_item_new_with_label(_("Sort Alphabetically Z-A"));
-	image = gtk_image_new_from_stock(GTK_STOCK_SORT_DESCENDING, GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_icon_name("view-sort-descending", GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
 	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &FavoritesPage::sort_descending, this);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 //-----------------------------------------------------------------------------
