@@ -39,6 +39,49 @@ using namespace WhiskerMenu;
 
 //-----------------------------------------------------------------------------
 
+#if GTK_CHECK_VERSION(3,20,0)
+
+static void grab_pointer(GdkWindow* window, guint32)
+{
+	GdkDisplay* display = gdk_display_get_default();
+	GdkSeat* seat = gdk_display_get_default_seat(display);
+	gdk_seat_grab(seat, window, GDK_SEAT_CAPABILITY_ALL_POINTING, true, NULL, NULL, NULL, NULL);
+}
+
+static void ungrab_pointer()
+{
+	GdkDisplay* display = gdk_display_get_default();
+	GdkSeat* seat = gdk_display_get_default_seat(display);
+	gdk_seat_ungrab(seat);
+}
+
+#else
+
+static void grab_pointer(GdkWindow* window, guint32 time)
+{
+	GdkDisplay* display = gdk_display_get_default();
+	GdkDeviceManager* device_manager = gdk_display_get_device_manager(display);
+	GdkDevice* device = gdk_device_manager_get_client_pointer(device_manager);
+	gdk_device_grab(device, window, GDK_OWNERSHIP_NONE, true,
+		GdkEventMask(
+			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+			GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
+		),
+		NULL, time);
+}
+
+static void ungrab_pointer()
+{
+	GdkDisplay* display = gdk_display_get_default();
+	GdkDeviceManager* device_manager = gdk_display_get_device_manager(display);
+	GdkDevice* device = gdk_device_manager_get_client_pointer(device_manager);
+	gdk_device_ungrab(device, gtk_get_current_event_time());
+}
+
+#endif
+
+//-----------------------------------------------------------------------------
+
 WhiskerMenu::Window::Window() :
 	m_window(NULL),
 	m_sidebar_size_group(NULL),
@@ -91,7 +134,7 @@ WhiskerMenu::Window::Window() :
 	gchar* username = g_markup_printf_escaped("<b><big>%s</big></b>", name);
 	m_username = GTK_LABEL(gtk_label_new(NULL));
 	gtk_label_set_markup(m_username, username);
-	gtk_misc_set_alignment(GTK_MISC(m_username), 0.0f, 0.5f);
+	gtk_widget_set_halign(GTK_WIDGET(m_username), GTK_ALIGN_START);
 	g_free(username);
 
 	// Create action buttons
@@ -144,49 +187,49 @@ WhiskerMenu::Window::Window() :
 	}
 
 	// Create box for packing children
-	m_vbox = GTK_BOX(gtk_vbox_new(false, 6));
+	m_vbox = GTK_BOX(gtk_box_new(GTK_ORIENTATION_VERTICAL, 6));
 	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(m_vbox));
 	gtk_container_set_border_width(GTK_CONTAINER(m_vbox), 2);
 
 	// Create box for packing commands
-	m_commands_align = GTK_ALIGNMENT(gtk_alignment_new(1, 0, 0, 0));
-	m_commands_box = GTK_BOX(gtk_hbox_new(false, 0));
+	m_commands_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
+	m_commands_spacer = gtk_label_new(NULL);
+	gtk_box_pack_start(m_commands_box, m_commands_spacer, true, true, 0);
 	for (int i = 0; i < 4; ++i)
 	{
 		gtk_box_pack_start(m_commands_box, m_commands_button[i], false, false, 0);
 	}
-	gtk_container_add(GTK_CONTAINER(m_commands_align), GTK_WIDGET(m_commands_box));
 
 	// Create box for packing username, commands, and resize widget
-	m_title_box = GTK_BOX(gtk_hbox_new(false, 0));
+	m_title_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
 	gtk_box_pack_start(m_vbox, GTK_WIDGET(m_title_box), false, false, 0);
 	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_profilepic->get_widget()), false, false, 0);
 	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_username), true, true, 0);
-	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_commands_align), false, false, 0);
+	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_commands_box), false, false, 0);
 	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_resizer->get_widget()), false, false, 0);
 
 	// Add search to layout
-	m_search_box = GTK_BOX(gtk_hbox_new(false, 6));
+	m_search_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6));
 	gtk_box_pack_start(m_vbox, GTK_WIDGET(m_search_box), false, true, 0);
 	gtk_box_pack_start(m_search_box, GTK_WIDGET(m_search_entry), true, true, 0);
 
 	// Create box for packing launcher pages and sidebar
-	m_contents_box = GTK_BOX(gtk_hbox_new(false, 6));
+	m_contents_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6));
 	gtk_box_pack_start(m_vbox, GTK_WIDGET(m_contents_box), true, true, 0);
 	gtk_box_pack_start(m_contents_box, m_search_results->get_widget(), true, true, 0);
 
 	// Create box for packing launcher pages
-	m_panels_box = GTK_BOX(gtk_vbox_new(false, 0));
+	m_panels_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_VERTICAL, 0));
 	gtk_box_pack_start(m_contents_box, GTK_WIDGET(m_panels_box), true, true, 0);
 	gtk_box_pack_start(m_panels_box, m_favorites->get_widget(), true, true, 0);
 	gtk_box_pack_start(m_panels_box, m_recent->get_widget(), true, true, 0);
 	gtk_box_pack_start(m_panels_box, m_applications->get_widget(), true, true, 0);
 
 	// Create box for packing sidebar
-	m_sidebar_buttons = GTK_BOX(gtk_vbox_new(false, 0));
+	m_sidebar_buttons = GTK_BOX(gtk_box_new(GTK_ORIENTATION_VERTICAL, 0));
 	gtk_box_pack_start(m_sidebar_buttons, GTK_WIDGET(m_favorites_button->get_button()), false, false, 0);
 	gtk_box_pack_start(m_sidebar_buttons, GTK_WIDGET(m_recent_button->get_button()), false, false, 0);
-	gtk_box_pack_start(m_sidebar_buttons, gtk_hseparator_new(), false, true, 0);
+	gtk_box_pack_start(m_sidebar_buttons, gtk_separator_new(GTK_ORIENTATION_HORIZONTAL), false, false, 0);
 
 	m_sidebar = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
 	gtk_box_pack_start(m_contents_box, GTK_WIDGET(m_sidebar), false, false, 0);
@@ -212,10 +255,8 @@ WhiskerMenu::Window::Window() :
 	gtk_window_set_default_size(m_window, m_geometry.width, m_geometry.height);
 
 	// Handle transparency
-	gtk_widget_set_app_paintable(GTK_WIDGET(m_sidebar_buttons), true);
-	g_signal_connect_slot(m_sidebar_buttons, "expose-event", &Window::on_expose_event, this);
 	gtk_widget_set_app_paintable(GTK_WIDGET(m_window), true);
-	g_signal_connect_slot(m_window, "expose-event", &Window::on_expose_event, this);
+	g_signal_connect_slot(m_window, "draw", &Window::on_draw_event, this);
 	g_signal_connect_slot(m_window, "screen-changed", &Window::on_screen_changed_event, this);
 	on_screen_changed_event(GTK_WIDGET(m_window), NULL);
 
@@ -253,7 +294,7 @@ WhiskerMenu::Window::~Window()
 
 void WhiskerMenu::Window::hide()
 {
-	gdk_pointer_ungrab(gtk_get_current_event_time());
+	ungrab_pointer();
 
 	// Hide command buttons to remove active border
 	for (int i = 0; i < 4; ++i)
@@ -363,13 +404,25 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 	else
 	{
 		GdkDisplay* display = gdk_display_get_default();
-		gdk_display_get_pointer(display, &screen, &parent_x, &parent_y, NULL);
+#if GTK_CHECK_VERSION(3,20,0)
+		GdkSeat* seat = gdk_display_get_default_seat(display);
+		GdkDevice* device = gdk_seat_get_pointer(seat);
+#else
+		GdkDeviceManager* device_manager = gdk_display_get_device_manager(display);
+		GdkDevice* device = gdk_device_manager_get_client_pointer(device_manager);
+#endif
+		gdk_device_get_position(device, &screen, &parent_x, &parent_y);
 	}
 
 	// Fetch screen geomtry
 	GdkRectangle monitor;
+#if GTK_CHECK_VERSION(3,22,0)
+	GdkMonitor* monitor_gdk = gdk_display_get_monitor_at_point(gdk_display_get_default(), parent_x, parent_y);
+	gdk_monitor_get_geometry(monitor_gdk, &monitor);
+#else
 	int monitor_num = gdk_screen_get_monitor_at_point(screen, parent_x, parent_y);
 	gdk_screen_get_monitor_geometry(screen, monitor_num, &monitor);
+#endif
 
 	// Prevent window from being larger than screen
 	if (m_geometry.width > monitor.width)
@@ -438,28 +491,28 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 		m_layout_left = !layout_left;
 		m_layout_commands_alternate = wm_settings->position_commands_alternate;
 
-		g_object_ref(m_commands_align);
+		g_object_ref(m_commands_box);
 		if (m_layout_commands_alternate)
 		{
 			if (!wm_settings->position_categories_alternate)
 			{
 				m_sidebar_size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 				gtk_size_group_add_widget(m_sidebar_size_group, GTK_WIDGET(m_sidebar));
-				gtk_size_group_add_widget(m_sidebar_size_group, GTK_WIDGET(m_commands_align));
+				gtk_size_group_add_widget(m_sidebar_size_group, GTK_WIDGET(m_commands_box));
 			}
 
-			gtk_container_remove(GTK_CONTAINER(m_title_box), GTK_WIDGET(m_commands_align));
-			gtk_box_pack_start(m_search_box, GTK_WIDGET(m_commands_align), false, false, 0);
+			gtk_container_remove(GTK_CONTAINER(m_title_box), GTK_WIDGET(m_commands_box));
+			gtk_box_pack_start(m_search_box, GTK_WIDGET(m_commands_box), false, false, 0);
 		}
 		else
 		{
 			g_object_unref(m_sidebar_size_group);
 			m_sidebar_size_group = NULL;
 
-			gtk_container_remove(GTK_CONTAINER(m_search_box), GTK_WIDGET(m_commands_align));
-			gtk_box_pack_start(m_title_box, GTK_WIDGET(m_commands_align), false, false, 0);
+			gtk_container_remove(GTK_CONTAINER(m_search_box), GTK_WIDGET(m_commands_box));
+			gtk_box_pack_start(m_title_box, GTK_WIDGET(m_commands_box), false, false, 0);
 		}
-		g_object_unref(m_commands_align);
+		g_object_unref(m_commands_box);
 	}
 	if ((layout_left && !wm_settings->position_categories_alternate)
 			|| (!layout_left && wm_settings->position_categories_alternate))
@@ -477,66 +530,66 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 		m_layout_left = layout_left;
 		if (m_layout_left && m_layout_commands_alternate)
 		{
-			gtk_misc_set_alignment(GTK_MISC(m_username), 0.0f, 0.5f);
+			gtk_widget_set_halign(GTK_WIDGET(m_username), GTK_ALIGN_START);
 
-			gtk_alignment_set(m_commands_align, 1, 0, 0, 0);
 			for (int i = 0; i < 4; ++i)
 			{
 				gtk_box_reorder_child(m_commands_box, m_commands_button[i], i);
 			}
+			gtk_box_reorder_child(m_commands_box, m_commands_spacer, 0);
 
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_profilepic->get_widget()), 0);
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_username), 1);
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_resizer->get_widget()), 2);
 
 			gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_search_entry), 0);
-			gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_commands_align), 1);
+			gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_commands_box), 1);
 		}
 		else if (m_layout_commands_alternate)
 		{
-			gtk_misc_set_alignment(GTK_MISC(m_username), 1.0f, 0.5f);
+			gtk_widget_set_halign(GTK_WIDGET(m_username), GTK_ALIGN_END);
 
-			gtk_alignment_set(m_commands_align, 0, 0, 0, 0);
 			for (int i = 0; i < 4; ++i)
 			{
 				gtk_box_reorder_child(m_commands_box, m_commands_button[i], 3 - i);
 			}
+			gtk_box_reorder_child(m_commands_box, m_commands_spacer, 4);
 
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_profilepic->get_widget()), 2);
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_username), 1);
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_resizer->get_widget()), 0);
 
 			gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_search_entry), 1);
-			gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_commands_align), 0);
+			gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_commands_box), 0);
 		}
 		else if (m_layout_left)
 		{
-			gtk_misc_set_alignment(GTK_MISC(m_username), 0.0f, 0.5f);
+			gtk_widget_set_halign(GTK_WIDGET(m_username), GTK_ALIGN_START);
 
-			gtk_alignment_set(m_commands_align, 1, 0, 0, 0);
 			for (int i = 0; i < 4; ++i)
 			{
 				gtk_box_reorder_child(m_commands_box, m_commands_button[i], i);
 			}
+			gtk_box_reorder_child(m_commands_box, m_commands_spacer, 0);
 
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_profilepic->get_widget()), 0);
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_username), 1);
-			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_commands_align), 2);
+			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_commands_box), 2);
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_resizer->get_widget()), 3);
 		}
 		else
 		{
-			gtk_misc_set_alignment(GTK_MISC(m_username), 1.0f, 0.5f);
+			gtk_widget_set_halign(GTK_WIDGET(m_username), GTK_ALIGN_END);
 
-			gtk_alignment_set(m_commands_align, 0, 0, 0, 0);
 			for (int i = 0; i < 4; ++i)
 			{
 				gtk_box_reorder_child(m_commands_box, m_commands_button[i], 3 - i);
 			}
+			gtk_box_reorder_child(m_commands_box, m_commands_spacer, 4);
 
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_profilepic->get_widget()), 3);
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_username), 2);
-			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_commands_align), 1);
+			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_commands_box), 1);
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_resizer->get_widget()), 0);
 		}
 	}
@@ -599,12 +652,7 @@ void WhiskerMenu::Window::save()
 
 void WhiskerMenu::Window::on_context_menu_destroyed()
 {
-	gdk_pointer_grab(gtk_widget_get_window(GTK_WIDGET(m_window)), true,
-			GdkEventMask(
-				GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-				GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-			),
-			NULL, NULL, gtk_get_current_event_time());
+	grab_pointer(gtk_widget_get_window(GTK_WIDGET(m_window)), gtk_get_current_event_time());
 }
 
 //-----------------------------------------------------------------------------
@@ -656,12 +704,7 @@ gboolean WhiskerMenu::Window::on_enter_notify_event(GtkWidget*, GdkEvent* event)
 		return false;
 	}
 
-	gdk_pointer_grab(gtk_widget_get_window(GTK_WIDGET(m_window)), true,
-			GdkEventMask(
-				GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-				GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-			),
-			NULL, NULL, crossing_event->time);
+	grab_pointer(gtk_widget_get_window(GTK_WIDGET(m_window)), crossing_event->time);
 
 	return false;
 }
@@ -677,12 +720,7 @@ gboolean WhiskerMenu::Window::on_leave_notify_event(GtkWidget*, GdkEvent* event)
 		return false;
 	}
 
-	gdk_pointer_grab(gtk_widget_get_window(GTK_WIDGET(m_window)), true,
-			GdkEventMask(
-				GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-				GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-			),
-			NULL, NULL, crossing_event->time);
+	grab_pointer(gtk_widget_get_window(GTK_WIDGET(m_window)), crossing_event->time);
 
 	return false;
 }
@@ -710,7 +748,7 @@ gboolean WhiskerMenu::Window::on_key_press_event(GtkWidget* widget, GdkEvent* ev
 	GdkEventKey* key_event = reinterpret_cast<GdkEventKey*>(event);
 
 	// Hide if escape is pressed and there is no text in search entry
-	if ( (key_event->keyval == GDK_Escape) && exo_str_is_empty(gtk_entry_get_text(m_search_entry)) )
+	if ( (key_event->keyval == GDK_KEY_Escape) && exo_str_is_empty(gtk_entry_get_text(m_search_entry)) )
 	{
 		hide();
 		return true;
@@ -787,12 +825,7 @@ gboolean WhiskerMenu::Window::on_map_event(GtkWidget*, GdkEvent*)
 	gtk_window_set_keep_above(m_window, true);
 
 	// Track mouse clicks outside of menu
-	gdk_pointer_grab(gtk_widget_get_window(GTK_WIDGET(m_window)), true,
-			GdkEventMask(
-				GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-				GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-			),
-			NULL, NULL, gtk_get_current_event_time());
+	grab_pointer(gtk_widget_get_window(GTK_WIDGET(m_window)), gtk_get_current_event_time());
 
 	// Focus search entry
 	gtk_widget_grab_focus(GTK_WIDGET(m_search_entry));
@@ -820,47 +853,50 @@ gboolean WhiskerMenu::Window::on_configure_event(GtkWidget*, GdkEvent* event)
 void WhiskerMenu::Window::on_screen_changed_event(GtkWidget* widget, GdkScreen*)
 {
 	GdkScreen* screen = gtk_widget_get_screen(widget);
-	GdkColormap* colormap = gdk_screen_get_rgba_colormap(screen);
-	if (!colormap || (wm_settings->menu_opacity == 100))
+	GdkVisual* visual = gdk_screen_get_rgba_visual(screen);
+	if (!visual || (wm_settings->menu_opacity == 100))
 	{
-		colormap = gdk_screen_get_system_colormap(screen);
+		visual = gdk_screen_get_system_visual(screen);
 		m_supports_alpha = false;
 	}
 	else
 	{
 		m_supports_alpha = true;
 	}
-	gtk_widget_set_colormap(widget, colormap);
+	gtk_widget_set_visual(widget, visual);
 }
 
 //-----------------------------------------------------------------------------
 
-gboolean WhiskerMenu::Window::on_expose_event(GtkWidget* widget, GdkEventExpose*)
+gboolean WhiskerMenu::Window::on_draw_event(GtkWidget* widget, cairo_t* cr)
 {
 	if (!gtk_widget_get_realized(widget))
 	{
 		gtk_widget_realize(widget);
 	}
 
-	GtkStyle* style = gtk_widget_get_style(widget);
-	if (style == NULL)
-	{
-		return false;
-	}
-	GdkColor color = style->bg[GTK_STATE_NORMAL];
+	GtkStyleContext* context = gtk_widget_get_style_context(widget);
+	const double width = gtk_widget_get_allocated_width(widget);
+	const double height = gtk_widget_get_allocated_height(widget);
 
-	cairo_t* cr = gdk_cairo_create(gtk_widget_get_window(widget));
 	if (m_supports_alpha)
 	{
-		cairo_set_source_rgba(cr, color.red / 65535.0, color.green / 65535.0, color.blue / 65535.0, wm_settings->menu_opacity / 100.0);
+		cairo_surface_t* background = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
+		cairo_t* cr_background = cairo_create(background);
+		cairo_set_operator(cr_background, CAIRO_OPERATOR_SOURCE);
+		gtk_render_background(context, cr_background, 0.0, 0.0, width, height);
+		cairo_destroy(cr_background);
+
+		cairo_set_source_surface(cr, background, 0.0, 0.0);
+		cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+		cairo_paint_with_alpha(cr, wm_settings->menu_opacity / 100.0);
+
+		cairo_surface_destroy(background);
 	}
 	else
 	{
-		cairo_set_source_rgb(cr, color.red / 65535.0, color.green / 65535.0, color.blue / 65535.0);
+		gtk_render_background(context, cr, 0.0, 0.0, width, height);
 	}
-	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
-	cairo_paint(cr);
-	cairo_destroy(cr);
 
 	return false;
 }
