@@ -82,7 +82,7 @@ private:
 	gboolean on_map_event(GtkWidget*, GdkEvent*);
 	gboolean on_configure_event(GtkWidget*, GdkEvent* event);
 	void on_screen_changed_event(GtkWidget* widget, GdkScreen* old_screen);
-	gboolean on_expose_event(GtkWidget* widget, GdkEventExpose* event);
+	gboolean on_draw_event(GtkWidget* widget, cairo_t* cr);
 	void favorites_toggled();
 	void recent_toggled();
 	void category_toggled();
@@ -104,7 +104,7 @@ private:
 	GtkLabel* m_username;
 	ResizerWidget* m_resizer;
 
-	GtkAlignment* m_commands_align;
+	GtkWidget* m_commands_spacer;
 	GtkWidget* m_commands_button[4];
 	gulong m_command_slots[4];
 
